#include <amxmodx>
#include <nvault>

// Constant.
const TASK_ID = 701

// Vault Name.
new const g_szVaultName[] = "played_time"
 
// Variables
new g_iVaultHandle
new g_iPlayedTime[MAX_PLAYERS+1]
 
// Forward allows register new natives.
public plugin_natives()
{
	register_native("ze_get_played_time", "@__native_get_played_time", 1)
}
 
// Forward called after server activation.
public plugin_init()
{
	// Load Plug-In.
	register_plugin("[ZE] Played Time", "1.1", "Raheem")
 
	// Client Commands.
	register_clcmd("say /played", "clcmd_ShowPlayedTime")
	register_clcmd("say_team /played", "clcmd_ShowPlayedTime")
}
 
// Forward called when player joined the server.
public client_putinserver(id)
{
	// Fake Client?
	if (is_user_bot(id) || is_user_hltv(id))
		return

	// Load played time.
	read_PlayedTime(id)

	// New task for calculate played time.
	set_task(1.0, "calc_PlayedTime", id, .flags = "b")
}
 
// Forward called when player disconnected from the server.
public client_disconnected(id, bool:drop)
{
	// Fake Client?
	if (is_user_bot(id) || is_user_hltv(id))
		return

	// Remove task.
	remove_task(id)

	// Save played time.
	write_PlayedTime(id)
}

public calc_PlayedTime(id)
{
	// +1 Second!
	g_iPlayedTime[id]++
}

// Hook called when player sent command "/played" in chat.
public clcmd_ShowPlayedTime(id)
{
	// Get played time.
	new iSeconds = get_user_time(id) + g_iPlayedTime[id]
 
	// Print colored message on chat for player.
	ze_colored_print(id, "!tPlayed Time:!y !g%02d!y:!g%02d!y:!g%02d!y", (iSeconds/3600), (iSeconds/60), (iSeconds%60))
}
 
/**
 * Private Functions.
 */
read_PlayedTime(id)
{
	// Local Variables.
	new szAuthID[34], szValue[16], iTimeStamp
 
	// Get SteamID of the player.
	get_user_authid(id, szAuthID, charsmax(szAuthID))
 
	// Open the nVault.
	g_iVaultHandle = nvault_open(g_szVaultName)
 
	// Error in opening Vault?
	if (g_iVaultHandle == INVALID_HANDLE)
	{
		// Pause Plug-In, Print error message on server console.
		set_fail_state("Error in opening the nVault !")
	}
 
	// Load string from the nVault?
	if (!nvault_lookup(g_iVaultHandle, szAuthID, szValue, charsmax(szValue), iTimeStamp))
	{
		g_iPlayedTime[id] = 0
	}
	else // Key exists?
	{
		// Convert a String to Integer.
		g_iPlayedTime[id] = str_to_num(szValue)
	}
 
	// Closes the nVault.
	nvault_close(g_iVaultHandle)
}
 
write_PlayedTime(id)
{
	// Strings.
	new szAuthID[34], szValue[16]
 
	// Get SteamID of the player.
	get_user_authid(id, szAuthID, charsmax(szAuthID))
 
	// Open the nVault.
	g_iVaultHandle = nvault_open(g_szVaultName)
 
	// Error in opening Vault?
	if (g_iVaultHandle == INVALID_HANDLE)
	{
		// Pause Plug-In, Print error message on server console.
		set_fail_state("Error in opening the nVault !")
	}

	// Get value from Array.
	new iValue = g_iPlayedTime[id]

	// The value > 0
	if (iValue > 0)
	{
		// Convert an Integer to String.
		num_to_str(iValue, szValue, charsmax(szValue))
	
		// Write played time on the Vault.
		nvault_set(g_iVaultHandle, szAuthID, szValue)
	
		// Reset Value.
		g_iPlayedTime[id] = 0
	}
 
	// Closes the nVault.
	nvault_close(g_iVaultHandle)
}
 
/**
 * Function of natives.
 */
@__native_get_played_time(id)
{
	// Player not connected?
	if (!is_user_connected(id))
	{
		// Print error on server console.
		log_error(AMX_ERR_NATIVE, "[ZE] Player not connected (%d)", id)
		return -1
	}
 
	return g_iPlayedTime[id] + get_user_time(id)
}